/************************************************************************
   Functions for reading from Midi file.

   Copyright (C) 1995 by R.Harmsen
 ***********************************************************************/

#include <string.h>
#include <stdlib.h>
#include "readmidi.h"

static int do_track (FILE *midifp, long tracklen, int track);

/************************************************************************

 ***********************************************************************/
int readmidi (FILE *midifp, rmd_callbackfunc callback)
{
   char buffer[64];
   long len;
   int tr, tracks;
   int ret;

   fseek(midifp, 0L, SEEK_SET);

   // Midi File Header
   if (fread(buffer, 8, 1, midifp) != 1)
      return RmdReadError;
   if (strncmp(buffer, "MThd", 4) != 0)
      return RmdNoMidiHeader;
   len = fourchars2long(buffer + 4);
   if (fread(buffer + 8, len, 1, midifp) != 1)
      return RmdReadError;
   callback(RmdHeader, buffer, 8 + len);

   tracks = twochars2int(buffer + 8 + 2);

   for (tr = 0; tr < tracks; tr++)
   {
      // Midi Tracks
      if (fread(buffer, 8, 1, midifp) != 1)
         return RmdReadError;
      if (strncmp(buffer, "MTrk", 4) != 0)
         return RmdInvalidTrackHeader;

      len = fourchars2long(buffer + 4);
      callback(RmdTrackHeader, buffer, 8);

      if ((ret = do_track(midifp, len, tr)) != RmdOK)
         return ret;
   }
   return RmdOK;
}


/************************************************************************

 ***********************************************************************/
static int do_track (FILE *midifp, long tracklen, int track)
{
   static unsigned char buffer[32 * 1024U];
   unsigned char *b = buffer;
   unsigned char *e = buffer + (unsigned long)(sizeof buffer);
   int bytes;
   long delta_time;

   if (tracklen > sizeof buffer)
      return RmdBufferTooSmall;

   if (fread(buffer, tracklen, 1, midifp) != 1)
      return RmdReadError;

   while (b < e)
   {
      delta_time = varlen2long(b, bytes);
      //@? Analyse midi-event here, and call callback function

      b += bytes;
   }
   return RmdOK;
}